@@ -15,6 +15,9 @@ pub struct ResolverProgress {
     time_to_print: Duration,
     printed: bool,
     deps_time: Duration,
+    /// Provides an escape hatch for machine with slow CPU for debugging and
+    /// testing Cargo itself.
+    /// See [rust-lang/cargo#6596](https://github.com/rust-lang/cargo/pull/6596) for more.
     #[cfg(debug_assertions)]
     slow_cpu_multiplier: u64,
 }
@@ -31,6 +34,9 @@ impl ResolverProgress {
             // Architectures that do not have a modern processor, hardware emulation, etc.
             // In the test code we have `slow_cpu_multiplier`, but that is not accessible here.
             #[cfg(debug_assertions)]
+            // ALLOWED: For testing cargo itself only. However, it was communicated as an public
+            // interface to other developers, so keep it as-is, shouldn't add `__CARGO` prefix.
+            #[allow(clippy::disallowed_methods)]
             slow_cpu_multiplier: std::env::var("CARGO_TEST_SLOW_CPU_MULTIPLIER")
                 .ok()
                 .and_then(|m| m.parse().ok())
@@ -119,11 +125,12 @@ impl ResolveBehavior {
         }
     }
 
-    pub fn to_manifest(&self) -> Option<String> {
+    pub fn to_manifest(&self) -> String {
         match self {
-            ResolveBehavior::V1 => None,
-            ResolveBehavior::V2 => Some("2".to_string()),
+            ResolveBehavior::V1 => "1",
+            ResolveBehavior::V2 => "2",
         }
+        .to_owned()
     }
 }
 
@@ -205,7 +212,7 @@ impl Ord for DepsFrame {
     }
 }
 
-/// Note that a `OrdSet` is used for the remaining dependencies that need
+/// Note that an `OrdSet` is used for the remaining dependencies that need
 /// activation. This set is sorted by how many candidates each dependency has.
 ///
 /// This helps us get through super constrained portions of the dependency
@@ -299,34 +306,22 @@ pub enum ConflictReason {
 
 impl ConflictReason {
     pub fn is_links(&self) -> bool {
-        if let ConflictReason::Links(_) = *self {
-            return true;
-        }
-        false
+        matches!(self, ConflictReason::Links(_))
     }
 
     pub fn is_missing_features(&self) -> bool {
-        if let ConflictReason::MissingFeatures(_) = *self {
-            return true;
-        }
-        false
+        matches!(self, ConflictReason::MissingFeatures(_))
     }
 
     pub fn is_required_dependency_as_features(&self) -> bool {
-        if let ConflictReason::RequiredDependencyAsFeature(_) = *self {
-            return true;
-        }
-        false
+        matches!(self, ConflictReason::RequiredDependencyAsFeature(_))
     }
 
     pub fn is_public_dependency(&self) -> bool {
-        if let ConflictReason::PublicDependency(_) = *self {
-            return true;
-        }
-        if let ConflictReason::PubliclyExports(_) = *self {
-            return true;
-        }
-        false
+        matches!(
+            self,
+            ConflictReason::PublicDependency(_) | ConflictReason::PubliclyExports(_)
+        )
     }
 }
 
