@@ -1,11 +1,42 @@
 use std::fmt;
 use std::io::prelude::*;
 
+use is_terminal::IsTerminal;
 use termcolor::Color::{Cyan, Green, Red, Yellow};
 use termcolor::{self, Color, ColorSpec, StandardStream, WriteColor};
 
 use crate::util::errors::CargoResult;
 
+pub enum TtyWidth {
+    NoTty,
+    Known(usize),
+    Guess(usize),
+}
+
+impl TtyWidth {
+    /// Returns the width of the terminal to use for diagnostics (which is
+    /// relayed to rustc via `--diagnostic-width`).
+    pub fn diagnostic_terminal_width(&self) -> Option<usize> {
+        // ALLOWED: For testing cargo itself only.
+        #[allow(clippy::disallowed_methods)]
+        if let Ok(width) = std::env::var("__CARGO_TEST_TTY_WIDTH_DO_NOT_USE_THIS") {
+            return Some(width.parse().unwrap());
+        }
+        match *self {
+            TtyWidth::NoTty | TtyWidth::Guess(_) => None,
+            TtyWidth::Known(width) => Some(width),
+        }
+    }
+
+    /// Returns the width used by progress bars for the tty.
+    pub fn progress_max_width(&self) -> Option<usize> {
+        match *self {
+            TtyWidth::NoTty => None,
+            TtyWidth::Known(width) | TtyWidth::Guess(width) => Some(width),
+        }
+    }
+}
+
 /// The requested verbosity of output.
 #[derive(Debug, Clone, Copy, PartialEq)]
 pub enum Verbosity {
@@ -71,13 +102,13 @@ impl Shell {
     /// Creates a new shell (color choice and verbosity), defaulting to 'auto' color and verbose
     /// output.
     pub fn new() -> Shell {
-        let auto = ColorChoice::CargoAuto.to_termcolor_color_choice();
+        let auto_clr = ColorChoice::CargoAuto;
         Shell {
             output: ShellOut::Stream {
-                stdout: StandardStream::stdout(auto),
-                stderr: StandardStream::stderr(auto),
+                stdout: StandardStream::stdout(auto_clr.to_termcolor_color_choice(Stream::Stdout)),
+                stderr: StandardStream::stderr(auto_clr.to_termcolor_color_choice(Stream::Stderr)),
                 color_choice: ColorChoice::CargoAuto,
-                stderr_tty: atty::is(atty::Stream::Stderr),
+                stderr_tty: std::io::stderr().is_terminal(),
             },
             verbosity: Verbosity::Verbose,
             needs_clear: false,
@@ -125,12 +156,12 @@ impl Shell {
     }
 
     /// Returns the width of the terminal in spaces, if any.
-    pub fn err_width(&self) -> Option<usize> {
+    pub fn err_width(&self) -> TtyWidth {
         match self.output {
             ShellOut::Stream {
                 stderr_tty: true, ..
             } => imp::stderr_width(),
-            _ => None,
+            _ => TtyWidth::NoTty,
         }
     }
 
@@ -160,10 +191,7 @@ impl Shell {
 
     /// Erase from cursor to end of line.
     pub fn err_erase_line(&mut self) {
-        if let ShellOut::Stream {
-            stderr_tty: true, ..
-        } = self.output
-        {
+        if self.err_supports_color() {
             imp::err_erase_line(self);
             self.needs_clear = false;
         }
@@ -275,9 +303,8 @@ impl Shell {
                 ),
             };
             *color_choice = cfg;
-            let choice = cfg.to_termcolor_color_choice();
-            *stdout = StandardStream::stdout(choice);
-            *stderr = StandardStream::stderr(choice);
+            *stdout = StandardStream::stdout(cfg.to_termcolor_color_choice(Stream::Stdout));
+            *stderr = StandardStream::stderr(cfg.to_termcolor_color_choice(Stream::Stderr));
         }
         Ok(())
     }
@@ -301,8 +328,37 @@ impl Shell {
         }
     }
 
-    /// Prints a message and translates ANSI escape code into console colors.
-    pub fn print_ansi(&mut self, message: &[u8]) -> CargoResult<()> {
+    pub fn out_supports_color(&self) -> bool {
+        match &self.output {
+            ShellOut::Write(_) => false,
+            ShellOut::Stream { stdout, .. } => stdout.supports_color(),
+        }
+    }
+
+    /// Write a styled fragment
+    ///
+    /// Caller is responsible for deciding whether [`Shell::verbosity`] is affects output.
+    pub fn write_stdout(
+        &mut self,
+        fragment: impl fmt::Display,
+        color: &ColorSpec,
+    ) -> CargoResult<()> {
+        self.output.write_stdout(fragment, color)
+    }
+
+    /// Write a styled fragment
+    ///
+    /// Caller is responsible for deciding whether [`Shell::verbosity`] is affects output.
+    pub fn write_stderr(
+        &mut self,
+        fragment: impl fmt::Display,
+        color: &ColorSpec,
+    ) -> CargoResult<()> {
+        self.output.write_stderr(fragment, color)
+    }
+
+    /// Prints a message to stderr and translates ANSI escape code into console colors.
+    pub fn print_ansi_stderr(&mut self, message: &[u8]) -> CargoResult<()> {
         if self.needs_clear {
             self.err_erase_line();
         }
@@ -317,9 +373,28 @@ impl Shell {
         Ok(())
     }
 
-    pub fn print_json<T: serde::ser::Serialize>(&mut self, obj: &T) {
-        let encoded = serde_json::to_string(&obj).unwrap();
+    /// Prints a message to stdout and translates ANSI escape code into console colors.
+    pub fn print_ansi_stdout(&mut self, message: &[u8]) -> CargoResult<()> {
+        if self.needs_clear {
+            self.err_erase_line();
+        }
+        #[cfg(windows)]
+        {
+            if let ShellOut::Stream { stdout, .. } = &mut self.output {
+                ::fwdansi::write_ansi(stdout, message)?;
+                return Ok(());
+            }
+        }
+        self.out().write_all(message)?;
+        Ok(())
+    }
+
+    pub fn print_json<T: serde::ser::Serialize>(&mut self, obj: &T) -> CargoResult<()> {
+        // Path may fail to serialize to JSON ...
+        let encoded = serde_json::to_string(&obj)?;
+        // ... but don't fail due to a closed pipe.
         drop(writeln!(self.out(), "{}", encoded));
+        Ok(())
     }
 }
 
@@ -372,6 +447,38 @@ impl ShellOut {
         Ok(())
     }
 
+    /// Write a styled fragment
+    fn write_stdout(&mut self, fragment: impl fmt::Display, color: &ColorSpec) -> CargoResult<()> {
+        match *self {
+            ShellOut::Stream { ref mut stdout, .. } => {
+                stdout.reset()?;
+                stdout.set_color(&color)?;
+                write!(stdout, "{}", fragment)?;
+                stdout.reset()?;
+            }
+            ShellOut::Write(ref mut w) => {
+                write!(w, "{}", fragment)?;
+            }
+        }
+        Ok(())
+    }
+
+    /// Write a styled fragment
+    fn write_stderr(&mut self, fragment: impl fmt::Display, color: &ColorSpec) -> CargoResult<()> {
+        match *self {
+            ShellOut::Stream { ref mut stderr, .. } => {
+                stderr.reset()?;
+                stderr.set_color(&color)?;
+                write!(stderr, "{}", fragment)?;
+                stderr.reset()?;
+            }
+            ShellOut::Write(ref mut w) => {
+                write!(w, "{}", fragment)?;
+            }
+        }
+        Ok(())
+    }
+
     /// Gets stdout as a `io::Write`.
     fn stdout(&mut self) -> &mut dyn Write {
         match *self {
@@ -391,12 +498,12 @@ impl ShellOut {
 
 impl ColorChoice {
     /// Converts our color choice to termcolor's version.
-    fn to_termcolor_color_choice(self) -> termcolor::ColorChoice {
+    fn to_termcolor_color_choice(self, stream: Stream) -> termcolor::ColorChoice {
         match self {
             ColorChoice::Always => termcolor::ColorChoice::Always,
             ColorChoice::Never => termcolor::ColorChoice::Never,
             ColorChoice::CargoAuto => {
-                if atty::is(atty::Stream::Stderr) {
+                if stream.is_terminal() {
                     termcolor::ColorChoice::Auto
                 } else {
                     termcolor::ColorChoice::Never
@@ -406,23 +513,37 @@ impl ColorChoice {
     }
 }
 
+enum Stream {
+    Stdout,
+    Stderr,
+}
+
+impl Stream {
+    fn is_terminal(self) -> bool {
+        match self {
+            Self::Stdout => std::io::stdout().is_terminal(),
+            Self::Stderr => std::io::stderr().is_terminal(),
+        }
+    }
+}
+
 #[cfg(unix)]
 mod imp {
-    use super::Shell;
+    use super::{Shell, TtyWidth};
     use std::mem;
 
-    pub fn stderr_width() -> Option<usize> {
+    pub fn stderr_width() -> TtyWidth {
         unsafe {
             let mut winsize: libc::winsize = mem::zeroed();
             // The .into() here is needed for FreeBSD which defines TIOCGWINSZ
             // as c_uint but ioctl wants c_ulong.
             if libc::ioctl(libc::STDERR_FILENO, libc::TIOCGWINSZ.into(), &mut winsize) < 0 {
-                return None;
+                return TtyWidth::NoTty;
             }
             if winsize.ws_col > 0 {
-                Some(winsize.ws_col as usize)
+                TtyWidth::Known(winsize.ws_col as usize)
             } else {
-                None
+                TtyWidth::NoTty
             }
         }
     }
@@ -438,37 +559,42 @@ mod imp {
 #[cfg(windows)]
 mod imp {
     use std::{cmp, mem, ptr};
-    use winapi::um::fileapi::*;
-    use winapi::um::handleapi::*;
-    use winapi::um::processenv::*;
-    use winapi::um::winbase::*;
-    use winapi::um::wincon::*;
-    use winapi::um::winnt::*;
 
-    pub(super) use super::default_err_erase_line as err_erase_line;
+    use windows_sys::core::PCSTR;
+    use windows_sys::Win32::Foundation::CloseHandle;
+    use windows_sys::Win32::Foundation::INVALID_HANDLE_VALUE;
+    use windows_sys::Win32::Storage::FileSystem::{
+        CreateFileA, FILE_SHARE_READ, FILE_SHARE_WRITE, OPEN_EXISTING,
+    };
+    use windows_sys::Win32::System::Console::{
+        GetConsoleScreenBufferInfo, GetStdHandle, CONSOLE_SCREEN_BUFFER_INFO, STD_ERROR_HANDLE,
+    };
+    use windows_sys::Win32::System::SystemServices::{GENERIC_READ, GENERIC_WRITE};
 
-    pub fn stderr_width() -> Option<usize> {
+    pub(super) use super::{default_err_erase_line as err_erase_line, TtyWidth};
+
+    pub fn stderr_width() -> TtyWidth {
         unsafe {
             let stdout = GetStdHandle(STD_ERROR_HANDLE);
             let mut csbi: CONSOLE_SCREEN_BUFFER_INFO = mem::zeroed();
             if GetConsoleScreenBufferInfo(stdout, &mut csbi) != 0 {
-                return Some((csbi.srWindow.Right - csbi.srWindow.Left) as usize);
+                return TtyWidth::Known((csbi.srWindow.Right - csbi.srWindow.Left) as usize);
             }
 
             // On mintty/msys/cygwin based terminals, the above fails with
             // INVALID_HANDLE_VALUE. Use an alternate method which works
             // in that case as well.
             let h = CreateFileA(
-                "CONOUT$\0".as_ptr() as *const CHAR,
+                "CONOUT$\0".as_ptr() as PCSTR,
                 GENERIC_READ | GENERIC_WRITE,
                 FILE_SHARE_READ | FILE_SHARE_WRITE,
                 ptr::null_mut(),
                 OPEN_EXISTING,
                 0,
-                ptr::null_mut(),
+                0,
             );
             if h == INVALID_HANDLE_VALUE {
-                return None;
+                return TtyWidth::NoTty;
             }
 
             let mut csbi: CONSOLE_SCREEN_BUFFER_INFO = mem::zeroed();
@@ -484,17 +610,21 @@ mod imp {
                 // resize the console correctly, but there's no reasonable way
                 // to detect which kind of terminal we are running in, or if
                 // GetConsoleScreenBufferInfo returns accurate information.
-                return Some(cmp::min(60, width));
+                return TtyWidth::Guess(cmp::min(60, width));
             }
-            None
+
+            TtyWidth::NoTty
         }
     }
 }
 
 #[cfg(windows)]
 fn default_err_erase_line(shell: &mut Shell) {
-    if let Some(max_width) = imp::stderr_width() {
-        let blank = " ".repeat(max_width);
-        drop(write!(shell.output.stderr(), "{}\r", blank));
+    match imp::stderr_width() {
+        TtyWidth::Known(max_width) | TtyWidth::Guess(max_width) => {
+            let blank = " ".repeat(max_width);
+            drop(write!(shell.output.stderr(), "{}\r", blank));
+        }
+        _ => (),
     }
 }
