@@ -1,7 +1,8 @@
 use crate::core::PackageId;
-use crate::sources::DirectorySource;
-use crate::sources::{GitSource, PathSource, RegistrySource, CRATES_IO_INDEX};
-use crate::util::{CanonicalUrl, CargoResult, Config, IntoUrl};
+use crate::sources::registry::CRATES_IO_HTTP_INDEX;
+use crate::sources::{DirectorySource, CRATES_IO_DOMAIN, CRATES_IO_INDEX, CRATES_IO_REGISTRY};
+use crate::sources::{GitSource, PathSource, RegistrySource};
+use crate::util::{config, CanonicalUrl, CargoResult, Config, IntoUrl};
 use log::trace;
 use serde::de;
 use serde::ser;
@@ -24,7 +25,7 @@ pub struct SourceId {
     inner: &'static SourceIdInner,
 }
 
-#[derive(PartialEq, Eq, Clone, Debug, Hash)]
+#[derive(Eq, Clone, Debug)]
 struct SourceIdInner {
     /// The source URL.
     url: Url,
@@ -38,6 +39,10 @@ struct SourceIdInner {
     /// WARNING: this is not always set for alt-registries when the name is
     /// not known.
     name: Option<String>,
+    /// Name of the alt registry in the `[registries]` table.
+    /// WARNING: this is not always set for alt-registries when the name is
+    /// not known.
+    alt_registry_key: Option<String>,
 }
 
 /// The possible kinds of code source. Along with `SourceIdInner`, this fully defines the
@@ -50,6 +55,8 @@ enum SourceKind {
     Path,
     /// A remote registry.
     Registry,
+    /// A sparse registry.
+    SparseRegistry,
     /// A local filesystem-based registry.
     LocalRegistry,
     /// A directory-based registry.
@@ -73,13 +80,22 @@ impl SourceId {
     /// Creates a `SourceId` object from the kind and URL.
     ///
     /// The canonical url will be calculated, but the precise field will not
-    fn new(kind: SourceKind, url: Url) -> CargoResult<SourceId> {
+    fn new(kind: SourceKind, url: Url, name: Option<&str>) -> CargoResult<SourceId> {
+        if kind == SourceKind::SparseRegistry {
+            // Sparse URLs are different because they store the kind prefix (sparse+)
+            // in the URL. This is because the prefix is necessary to differentiate
+            // from regular registries (git-based). The sparse+ prefix is included
+            // everywhere, including user-facing locations such as the `config.toml`
+            // file that defines the registry, or whenever Cargo displays it to the user.
+            assert!(url.as_str().starts_with("sparse+"));
+        }
         let source_id = SourceId::wrap(SourceIdInner {
             kind,
             canonical_url: CanonicalUrl::new(&url)?,
             url,
             precise: None,
-            name: None,
+            name: name.map(|n| n.into()),
+            alt_registry_key: None,
         });
         Ok(source_id)
     }
@@ -94,6 +110,14 @@ impl SourceId {
         SourceId { inner }
     }
 
+    fn remote_source_kind(url: &Url) -> SourceKind {
+        if url.as_str().starts_with("sparse+") {
+            SourceKind::SparseRegistry
+        } else {
+            SourceKind::Registry
+        }
+    }
+
     /// Parses a source URL and returns the corresponding ID.
     ///
     /// ## Example
@@ -132,12 +156,17 @@ impl SourceId {
             }
             "registry" => {
                 let url = url.into_url()?;
-                Ok(SourceId::new(SourceKind::Registry, url)?
+                Ok(SourceId::new(SourceKind::Registry, url, None)?
+                    .with_precise(Some("locked".to_string())))
+            }
+            "sparse" => {
+                let url = string.into_url()?;
+                Ok(SourceId::new(SourceKind::SparseRegistry, url, None)?
                     .with_precise(Some("locked".to_string())))
             }
             "path" => {
                 let url = url.into_url()?;
-                SourceId::new(SourceKind::Path, url)
+                SourceId::new(SourceKind::Path, url, None)
             }
             kind => Err(anyhow::format_err!("unsupported source protocol: {}", kind)),
         }
@@ -155,51 +184,95 @@ impl SourceId {
     /// `path`: an absolute path.
     pub fn for_path(path: &Path) -> CargoResult<SourceId> {
         let url = path.into_url()?;
-        SourceId::new(SourceKind::Path, url)
+        SourceId::new(SourceKind::Path, url, None)
     }
 
     /// Creates a `SourceId` from a Git reference.
     pub fn for_git(url: &Url, reference: GitReference) -> CargoResult<SourceId> {
-        SourceId::new(SourceKind::Git(reference), url.clone())
+        SourceId::new(SourceKind::Git(reference), url.clone(), None)
     }
 
-    /// Creates a SourceId from a registry URL.
+    /// Creates a SourceId from a remote registry URL when the registry name
+    /// cannot be determined, e.g. a user passes `--index` directly from CLI.
+    ///
+    /// Use [`SourceId::for_alt_registry`] if a name can provided, which
+    /// generates better messages for cargo.
     pub fn for_registry(url: &Url) -> CargoResult<SourceId> {
-        SourceId::new(SourceKind::Registry, url.clone())
+        let kind = Self::remote_source_kind(url);
+        SourceId::new(kind, url.to_owned(), None)
+    }
+
+    /// Creates a `SourceId` from a remote registry URL with given name.
+    pub fn for_alt_registry(url: &Url, name: &str) -> CargoResult<SourceId> {
+        let kind = Self::remote_source_kind(url);
+        SourceId::new(kind, url.to_owned(), Some(name))
     }
 
     /// Creates a SourceId from a local registry path.
     pub fn for_local_registry(path: &Path) -> CargoResult<SourceId> {
         let url = path.into_url()?;
-        SourceId::new(SourceKind::LocalRegistry, url)
+        SourceId::new(SourceKind::LocalRegistry, url, None)
     }
 
     /// Creates a `SourceId` from a directory path.
     pub fn for_directory(path: &Path) -> CargoResult<SourceId> {
         let url = path.into_url()?;
-        SourceId::new(SourceKind::Directory, url)
+        SourceId::new(SourceKind::Directory, url, None)
     }
 
     /// Returns the `SourceId` corresponding to the main repository.
     ///
     /// This is the main cargo registry by default, but it can be overridden in
-    /// a `.cargo/config`.
+    /// a `.cargo/config.toml`.
     pub fn crates_io(config: &Config) -> CargoResult<SourceId> {
         config.crates_io_source_id(|| {
             config.check_registry_index_not_set()?;
             let url = CRATES_IO_INDEX.into_url().unwrap();
-            SourceId::for_registry(&url)
+            SourceId::new(SourceKind::Registry, url, Some(CRATES_IO_REGISTRY))
         })
     }
 
+    /// Returns the `SourceId` corresponding to the main repository, using the
+    /// sparse HTTP index if allowed.
+    pub fn crates_io_maybe_sparse_http(config: &Config) -> CargoResult<SourceId> {
+        if Self::crates_io_is_sparse(config)? {
+            config.check_registry_index_not_set()?;
+            let url = CRATES_IO_HTTP_INDEX.into_url().unwrap();
+            SourceId::new(SourceKind::SparseRegistry, url, Some(CRATES_IO_REGISTRY))
+        } else {
+            Self::crates_io(config)
+        }
+    }
+
+    /// Returns whether to access crates.io over the sparse protocol.
+    pub fn crates_io_is_sparse(config: &Config) -> CargoResult<bool> {
+        let proto: Option<config::Value<String>> = config.get("registries.crates-io.protocol")?;
+        let is_sparse = match proto.as_ref().map(|v| v.val.as_str()) {
+            Some("sparse") => true,
+            Some("git") => false,
+            Some(unknown) => anyhow::bail!(
+                "unsupported registry protocol `{unknown}` (defined in {})",
+                proto.as_ref().unwrap().definition
+            ),
+            None => true,
+        };
+        Ok(is_sparse)
+    }
+
+    /// Gets the `SourceId` associated with given name of the remote registry.
     pub fn alt_registry(config: &Config, key: &str) -> CargoResult<SourceId> {
+        if key == CRATES_IO_REGISTRY {
+            return Self::crates_io(config);
+        }
         let url = config.get_registry_index(key)?;
+        let kind = Self::remote_source_kind(&url);
         Ok(SourceId::wrap(SourceIdInner {
-            kind: SourceKind::Registry,
+            kind,
             canonical_url: CanonicalUrl::new(&url)?,
             url,
             precise: None,
             name: Some(key.to_string()),
+            alt_registry_key: Some(key.to_string()),
         }))
     }
 
@@ -215,23 +288,34 @@ impl SourceId {
     }
 
     pub fn display_index(self) -> String {
-        if self.is_default_registry() {
-            "crates.io index".to_string()
+        if self.is_crates_io() {
+            format!("{} index", CRATES_IO_DOMAIN)
         } else {
-            format!("`{}` index", url_display(self.url()))
+            format!("`{}` index", self.display_registry_name())
         }
     }
 
     pub fn display_registry_name(self) -> String {
-        if self.is_default_registry() {
-            "crates.io".to_string()
+        if self.is_crates_io() {
+            CRATES_IO_REGISTRY.to_string()
         } else if let Some(name) = &self.inner.name {
             name.clone()
+        } else if self.precise().is_some() {
+            // We remove `precise` here to retrieve an permissive version of
+            // `SourceIdInner`, which may contain the registry name.
+            self.with_precise(None).display_registry_name()
         } else {
             url_display(self.url())
         }
     }
 
+    /// Gets the name of the remote registry as defined in the `[registries]` table.
+    /// WARNING: alt registries that come from Cargo.lock, or --index will
+    /// not have a name.
+    pub fn alt_registry_key(&self) -> Option<&str> {
+        self.inner.alt_registry_key.as_deref()
+    }
+
     /// Returns `true` if this source is from a filesystem path.
     pub fn is_path(self) -> bool {
         self.inner.kind == SourceKind::Path
@@ -250,16 +334,24 @@ impl SourceId {
     pub fn is_registry(self) -> bool {
         matches!(
             self.inner.kind,
-            SourceKind::Registry | SourceKind::LocalRegistry
+            SourceKind::Registry | SourceKind::SparseRegistry | SourceKind::LocalRegistry
         )
     }
 
+    /// Returns `true` if this source is from a sparse registry.
+    pub fn is_sparse(self) -> bool {
+        matches!(self.inner.kind, SourceKind::SparseRegistry)
+    }
+
     /// Returns `true` if this source is a "remote" registry.
     ///
     /// "remote" may also mean a file URL to a git index, so it is not
     /// necessarily "remote". This just means it is not `local-registry`.
     pub fn is_remote_registry(self) -> bool {
-        matches!(self.inner.kind, SourceKind::Registry)
+        matches!(
+            self.inner.kind,
+            SourceKind::Registry | SourceKind::SparseRegistry
+        )
     }
 
     /// Returns `true` if this source from a Git repository.
@@ -283,11 +375,9 @@ impl SourceId {
                 };
                 Ok(Box::new(PathSource::new(&path, self, config)))
             }
-            SourceKind::Registry => Ok(Box::new(RegistrySource::remote(
-                self,
-                yanked_whitelist,
-                config,
-            ))),
+            SourceKind::Registry | SourceKind::SparseRegistry => Ok(Box::new(
+                RegistrySource::remote(self, yanked_whitelist, config)?,
+            )),
             SourceKind::LocalRegistry => {
                 let path = match self.inner.url.to_file_path() {
                     Ok(p) => p,
@@ -332,12 +422,13 @@ impl SourceId {
     }
 
     /// Returns `true` if the remote registry is the standard <https://crates.io>.
-    pub fn is_default_registry(self) -> bool {
+    pub fn is_crates_io(self) -> bool {
         match self.inner.kind {
-            SourceKind::Registry => {}
+            SourceKind::Registry | SourceKind::SparseRegistry => {}
             _ => return false,
         }
-        self.inner.url.as_str() == CRATES_IO_INDEX
+        let url = self.inner.url.as_str();
+        url == CRATES_IO_INDEX || url == CRATES_IO_HTTP_INDEX || is_overridden_crates_io_url(url)
     }
 
     /// Hashes `self`.
@@ -463,7 +554,9 @@ impl fmt::Display for SourceId {
                 Ok(())
             }
             SourceKind::Path => write!(f, "{}", url_display(&self.inner.url)),
-            SourceKind::Registry => write!(f, "registry `{}`", url_display(&self.inner.url)),
+            SourceKind::Registry | SourceKind::SparseRegistry => {
+                write!(f, "registry `{}`", self.display_registry_name())
+            }
             SourceKind::LocalRegistry => write!(f, "registry `{}`", url_display(&self.inner.url)),
             SourceKind::Directory => write!(f, "dir {}", url_display(&self.inner.url)),
         }
@@ -483,6 +576,29 @@ impl Hash for SourceId {
     }
 }
 
+impl Hash for SourceIdInner {
+    /// The hash of `SourceIdInner` is used to retrieve its interned value. We
+    /// only care about fields that make `SourceIdInner` unique, which are:
+    ///
+    /// - `kind`
+    /// - `precise`
+    /// - `canonical_url`
+    fn hash<S: hash::Hasher>(&self, into: &mut S) {
+        self.kind.hash(into);
+        self.precise.hash(into);
+        self.canonical_url.hash(into);
+    }
+}
+
+impl PartialEq for SourceIdInner {
+    /// This implementation must be synced with [`SourceIdInner::hash`].
+    fn eq(&self, other: &Self) -> bool {
+        self.kind == other.kind
+            && self.precise == other.precise
+            && self.canonical_url == other.canonical_url
+    }
+}
+
 // forward to `Ord`
 impl PartialOrd for SourceKind {
     fn partial_cmp(&self, other: &SourceKind) -> Option<Ordering> {
@@ -537,7 +653,7 @@ impl PartialOrd for SourceKind {
 // (1.53) #9397 was created to fix the regression introduced by #9133 relative
 // to the current stable (1.51).
 //
-// That's all a long winded way of saying "it's wierd that git deps hash first
+// That's all a long winded way of saying "it's weird that git deps hash first
 // and are sorted last, but it's the way it is right now". The author of this
 // comment chose to handwrite the `Ord` implementation instead of the `Hash`
 // implementation, but it's only required that at most one of them is
@@ -554,6 +670,10 @@ impl Ord for SourceKind {
             (SourceKind::Registry, _) => Ordering::Less,
             (_, SourceKind::Registry) => Ordering::Greater,
 
+            (SourceKind::SparseRegistry, SourceKind::SparseRegistry) => Ordering::Equal,
+            (SourceKind::SparseRegistry, _) => Ordering::Less,
+            (_, SourceKind::SparseRegistry) => Ordering::Greater,
+
             (SourceKind::LocalRegistry, SourceKind::LocalRegistry) => Ordering::Equal,
             (SourceKind::LocalRegistry, _) => Ordering::Less,
             (_, SourceKind::LocalRegistry) => Ordering::Greater,
@@ -567,26 +687,6 @@ impl Ord for SourceKind {
     }
 }
 
-// This is a test that the hash of the `SourceId` for crates.io is a well-known
-// value.
-//
-// Note that the hash value matches what the crates.io source id has hashed
-// since long before Rust 1.30. We strive to keep this value the same across
-// versions of Cargo because changing it means that users will need to
-// redownload the index and all crates they use when using a new Cargo version.
-//
-// This isn't to say that this hash can *never* change, only that when changing
-// this it should be explicitly done. If this hash changes accidentally and
-// you're able to restore the hash to its original value, please do so!
-// Otherwise please just leave a comment in your PR as to why the hash value is
-// changing and why the old value can't be easily preserved.
-#[test]
-fn test_cratesio_hash() {
-    let config = Config::default().unwrap();
-    let crates_io = SourceId::crates_io(&config).unwrap();
-    assert_eq!(crate::util::hex::short_hash(&crates_io), "1ecc6299db9ec823");
-}
-
 /// A `Display`able view into a `SourceId` that will write it as a url
 pub struct SourceIdAsUrl<'a> {
     inner: &'a SourceIdInner,
@@ -619,7 +719,17 @@ impl<'a> fmt::Display for SourceIdAsUrl<'a> {
                 kind: SourceKind::Registry,
                 ref url,
                 ..
-            } => write!(f, "registry+{}", url),
+            } => {
+                write!(f, "registry+{url}")
+            }
+            SourceIdInner {
+                kind: SourceKind::SparseRegistry,
+                ref url,
+                ..
+            } => {
+                // Sparse registry URL already includes the `sparse+` prefix
+                write!(f, "{url}")
+            }
             SourceIdInner {
                 kind: SourceKind::LocalRegistry,
                 ref url,
@@ -664,21 +774,118 @@ impl<'a> fmt::Display for PrettyRef<'a> {
 #[cfg(test)]
 mod tests {
     use super::{GitReference, SourceId, SourceKind};
-    use crate::util::IntoUrl;
+    use crate::util::{Config, IntoUrl};
 
     #[test]
     fn github_sources_equal() {
         let loc = "https://github.com/foo/bar".into_url().unwrap();
         let default = SourceKind::Git(GitReference::DefaultBranch);
-        let s1 = SourceId::new(default.clone(), loc).unwrap();
+        let s1 = SourceId::new(default.clone(), loc, None).unwrap();
 
         let loc = "git://github.com/foo/bar".into_url().unwrap();
-        let s2 = SourceId::new(default, loc.clone()).unwrap();
+        let s2 = SourceId::new(default, loc.clone(), None).unwrap();
 
         assert_eq!(s1, s2);
 
         let foo = SourceKind::Git(GitReference::Branch("foo".to_string()));
-        let s3 = SourceId::new(foo, loc).unwrap();
+        let s3 = SourceId::new(foo, loc, None).unwrap();
         assert_ne!(s1, s3);
     }
+
+    // This is a test that the hash of the `SourceId` for crates.io is a well-known
+    // value.
+    //
+    // Note that the hash value matches what the crates.io source id has hashed
+    // since long before Rust 1.30. We strive to keep this value the same across
+    // versions of Cargo because changing it means that users will need to
+    // redownload the index and all crates they use when using a new Cargo version.
+    //
+    // This isn't to say that this hash can *never* change, only that when changing
+    // this it should be explicitly done. If this hash changes accidentally and
+    // you're able to restore the hash to its original value, please do so!
+    // Otherwise please just leave a comment in your PR as to why the hash value is
+    // changing and why the old value can't be easily preserved.
+    //
+    // The hash value depends on endianness and bit-width, so we only run this test on
+    // little-endian 64-bit CPUs (such as x86-64 and ARM64) where it matches the
+    // well-known value.
+    #[test]
+    #[cfg(all(target_endian = "little", target_pointer_width = "64"))]
+    fn test_cratesio_hash() {
+        let config = Config::default().unwrap();
+        let crates_io = SourceId::crates_io(&config).unwrap();
+        assert_eq!(crate::util::hex::short_hash(&crates_io), "1ecc6299db9ec823");
+    }
+
+    // See the comment in `test_cratesio_hash`.
+    //
+    // Only test on non-Windows as paths on Windows will get different hashes.
+    #[test]
+    #[cfg(all(target_endian = "little", target_pointer_width = "64", not(windows)))]
+    fn test_stable_hash() {
+        use std::hash::Hasher;
+        use std::path::Path;
+
+        let gen_hash = |source_id: SourceId| {
+            let mut hasher = std::collections::hash_map::DefaultHasher::new();
+            source_id.stable_hash(Path::new("/tmp/ws"), &mut hasher);
+            hasher.finish()
+        };
+
+        let url = "https://my-crates.io".into_url().unwrap();
+        let source_id = SourceId::for_registry(&url).unwrap();
+        assert_eq!(gen_hash(source_id), 18108075011063494626);
+        assert_eq!(crate::util::hex::short_hash(&source_id), "fb60813d6cb8df79");
+
+        let url = "https://your-crates.io".into_url().unwrap();
+        let source_id = SourceId::for_alt_registry(&url, "alt").unwrap();
+        assert_eq!(gen_hash(source_id), 12862859764592646184);
+        assert_eq!(crate::util::hex::short_hash(&source_id), "09c10fd0cbd74bce");
+
+        let url = "sparse+https://my-crates.io".into_url().unwrap();
+        let source_id = SourceId::for_registry(&url).unwrap();
+        assert_eq!(gen_hash(source_id), 8763561830438022424);
+        assert_eq!(crate::util::hex::short_hash(&source_id), "d1ea0d96f6f759b5");
+
+        let url = "sparse+https://your-crates.io".into_url().unwrap();
+        let source_id = SourceId::for_alt_registry(&url, "alt").unwrap();
+        assert_eq!(gen_hash(source_id), 5159702466575482972);
+        assert_eq!(crate::util::hex::short_hash(&source_id), "135d23074253cb78");
+
+        let url = "file:///tmp/ws/crate".into_url().unwrap();
+        let source_id = SourceId::for_git(&url, GitReference::DefaultBranch).unwrap();
+        assert_eq!(gen_hash(source_id), 15332537265078583985);
+        assert_eq!(crate::util::hex::short_hash(&source_id), "73a808694abda756");
+
+        let path = Path::new("/tmp/ws/crate");
+
+        let source_id = SourceId::for_local_registry(path).unwrap();
+        assert_eq!(gen_hash(source_id), 18446533307730842837);
+        assert_eq!(crate::util::hex::short_hash(&source_id), "52a84cc73f6fd48b");
+
+        let source_id = SourceId::for_path(path).unwrap();
+        assert_eq!(gen_hash(source_id), 8764714075439899829);
+        assert_eq!(crate::util::hex::short_hash(&source_id), "e1ddd48578620fc1");
+
+        let source_id = SourceId::for_directory(path).unwrap();
+        assert_eq!(gen_hash(source_id), 17459999773908528552);
+        assert_eq!(crate::util::hex::short_hash(&source_id), "6568fe2c2fab5bfe");
+    }
+
+    #[test]
+    fn serde_roundtrip() {
+        let url = "sparse+https://my-crates.io/".into_url().unwrap();
+        let source_id = SourceId::for_registry(&url).unwrap();
+        let formatted = format!("{}", source_id.as_url());
+        let deserialized = SourceId::from_url(&formatted).unwrap();
+        assert_eq!(formatted, "sparse+https://my-crates.io/");
+        assert_eq!(source_id, deserialized);
+    }
+}
+
+/// Check if `url` equals to the overridden crates.io URL.
+// ALLOWED: For testing Cargo itself only.
+#[allow(clippy::disallowed_methods)]
+fn is_overridden_crates_io_url(url: &str) -> bool {
+    std::env::var("__CARGO_TEST_CRATES_IO_URL_DO_NOT_USE_THIS").map_or(false, |v| v == url)
 }
