@@ -4,7 +4,7 @@ use std::path::{Path, PathBuf};
 use std::task::Poll;
 
 use crate::core::source::MaybePackage;
-use crate::core::{Dependency, Package, PackageId, Source, SourceId, Summary};
+use crate::core::{Dependency, Package, PackageId, QueryKind, Source, SourceId, Summary};
 use crate::ops;
 use crate::util::{internal, CargoResult, Config};
 use anyhow::Context as _;
@@ -150,7 +150,7 @@ impl<'cfg> PathSource<'cfg> {
             }
         };
 
-        let mut filter = |path: &Path, is_dir: bool| {
+        let filter = |path: &Path, is_dir: bool| {
             let relative_path = match path.strip_prefix(root) {
                 Ok(p) => p,
                 Err(_) => return false,
@@ -169,10 +169,10 @@ impl<'cfg> PathSource<'cfg> {
         // Attempt Git-prepopulate only if no `include` (see rust-lang/cargo#4135).
         if no_include_option {
             if let Some(repo) = git_repo {
-                return self.list_files_git(pkg, &repo, &mut filter);
+                return self.list_files_git(pkg, &repo, &filter);
             }
         }
-        self.list_files_walk(pkg, &mut filter)
+        self.list_files_walk(pkg, &filter)
     }
 
     /// Returns `Some(git2::Repository)` if found sibling `Cargo.toml` and `.git`
@@ -222,7 +222,7 @@ impl<'cfg> PathSource<'cfg> {
         &self,
         pkg: &Package,
         repo: &git2::Repository,
-        filter: &mut dyn FnMut(&Path, bool) -> bool,
+        filter: &dyn Fn(&Path, bool) -> bool,
     ) -> CargoResult<Vec<PathBuf>> {
         warn!("list_files_git {}", pkg.package_id());
         let index = repo.index()?;
@@ -376,7 +376,7 @@ impl<'cfg> PathSource<'cfg> {
     fn list_files_walk(
         &self,
         pkg: &Package,
-        filter: &mut dyn FnMut(&Path, bool) -> bool,
+        filter: &dyn Fn(&Path, bool) -> bool,
     ) -> CargoResult<Vec<PathBuf>> {
         let mut ret = Vec::new();
         self.walk(pkg.root(), &mut ret, true, filter)?;
@@ -388,7 +388,7 @@ impl<'cfg> PathSource<'cfg> {
         path: &Path,
         ret: &mut Vec<PathBuf>,
         is_root: bool,
-        filter: &mut dyn FnMut(&Path, bool) -> bool,
+        filter: &dyn Fn(&Path, bool) -> bool,
     ) -> CargoResult<()> {
         let walkdir = WalkDir::new(path)
             .follow_links(true)
@@ -432,7 +432,11 @@ impl<'cfg> PathSource<'cfg> {
                     self.config.shell().warn(err)?;
                 }
                 Err(err) => match err.path() {
-                    // If the error occurs with a path, simply recover from it.
+                    // If an error occurs with a path, filter it again.
+                    // If it is excluded, Just ignore it in this case.
+                    // See issue rust-lang/cargo#10917
+                    Some(path) if !filter(path, path.is_dir()) => {}
+                    // Otherwise, simply recover from it.
                     // Don't worry about error skipping here, the callers would
                     // still hit the IO error if they do access it thereafter.
                     Some(path) => ret.push(path.to_path_buf()),
@@ -497,24 +501,21 @@ impl<'cfg> Debug for PathSource<'cfg> {
 }
 
 impl<'cfg> Source for PathSource<'cfg> {
-    fn query(&mut self, dep: &Dependency, f: &mut dyn FnMut(Summary)) -> Poll<CargoResult<()>> {
-        self.update()?;
-        for s in self.packages.iter().map(|p| p.summary()) {
-            if dep.matches(s) {
-                f(s.clone())
-            }
-        }
-        Poll::Ready(Ok(()))
-    }
-
-    fn fuzzy_query(
+    fn query(
         &mut self,
-        _dep: &Dependency,
+        dep: &Dependency,
+        kind: QueryKind,
         f: &mut dyn FnMut(Summary),
     ) -> Poll<CargoResult<()>> {
         self.update()?;
         for s in self.packages.iter().map(|p| p.summary()) {
-            f(s.clone())
+            let matched = match kind {
+                QueryKind::Exact => dep.matches(s),
+                QueryKind::Fuzzy => true,
+            };
+            if matched {
+                f(s.clone())
+            }
         }
         Poll::Ready(Ok(()))
     }
@@ -562,8 +563,8 @@ impl<'cfg> Source for PathSource<'cfg> {
 
     fn add_to_yanked_whitelist(&mut self, _pkgs: &[PackageId]) {}
 
-    fn is_yanked(&mut self, _pkg: PackageId) -> CargoResult<bool> {
-        Ok(false)
+    fn is_yanked(&mut self, _pkg: PackageId) -> Poll<CargoResult<bool>> {
+        Poll::Ready(Ok(false))
     }
 
     fn block_until_ready(&mut self) -> CargoResult<()> {
@@ -573,4 +574,8 @@ impl<'cfg> Source for PathSource<'cfg> {
     fn invalidate_cache(&mut self) {
         // Path source has no local cache.
     }
+
+    fn set_quiet(&mut self, _quiet: bool) {
+        // Path source does not display status
+    }
 }
