@@ -66,7 +66,6 @@
 //! details like invalidating caches and whatnot which are handled below, but
 //! hopefully those are more obvious inline in the code itself.
 
-use crate::core::dependency::Dependency;
 use crate::core::{PackageId, SourceId, Summary};
 use crate::sources::registry::{LoadResponse, RegistryData, RegistryPackage, INDEX_V_MAX};
 use crate::util::interning::InternedString;
@@ -76,26 +75,25 @@ use cargo_util::{paths, registry::make_dep_path};
 use log::{debug, info};
 use semver::Version;
 use std::collections::{HashMap, HashSet};
-use std::convert::TryInto;
 use std::fs;
 use std::io::ErrorKind;
 use std::path::Path;
 use std::str;
-use std::task::Poll;
+use std::task::{ready, Poll};
 
 /// Crates.io treats hyphen and underscores as interchangeable, but the index and old Cargo do not.
 /// Therefore, the index must store uncanonicalized version of the name so old Cargo's can find it.
 /// This loop tries all possible combinations of switching hyphen and underscores to find the
 /// uncanonicalized one. As all stored inputs have the correct spelling, we start with the spelling
 /// as-provided.
-struct UncanonicalizedIter<'s> {
+pub struct UncanonicalizedIter<'s> {
     input: &'s str,
     num_hyphen_underscore: u32,
     hyphen_combination_num: u16,
 }
 
 impl<'s> UncanonicalizedIter<'s> {
-    fn new(input: &'s str) -> Self {
+    pub fn new(input: &'s str) -> Self {
         let num_hyphen_underscore = input.chars().filter(|&c| c == '_' || c == '-').count() as u32;
         UncanonicalizedIter {
             input,
@@ -268,11 +266,8 @@ impl<'cfg> RegistryIndex<'cfg> {
     /// Returns the hash listed for a specified `PackageId`.
     pub fn hash(&mut self, pkg: PackageId, load: &mut dyn RegistryData) -> Poll<CargoResult<&str>> {
         let req = OptVersionReq::exact(pkg.version());
-        let summary = self.summaries(pkg.name(), &req, load)?;
-        let summary = match summary {
-            Poll::Ready(mut summary) => summary.next(),
-            Poll::Pending => return Poll::Pending,
-        };
+        let summary = self.summaries(&pkg.name(), &req, load)?;
+        let summary = ready!(summary).next();
         Poll::Ready(Ok(summary
             .ok_or_else(|| internal(format!("no hash listed for {}", pkg)))?
             .summary
@@ -289,7 +284,7 @@ impl<'cfg> RegistryIndex<'cfg> {
     /// though since this method is called quite a lot on null builds in Cargo.
     pub fn summaries<'a, 'b>(
         &'a mut self,
-        name: InternedString,
+        name: &str,
         req: &'b OptVersionReq,
         load: &mut dyn RegistryData,
     ) -> Poll<CargoResult<impl Iterator<Item = &'a IndexSummary> + 'b>>
@@ -303,10 +298,8 @@ impl<'cfg> RegistryIndex<'cfg> {
         // has run previously this will parse a Cargo-specific cache file rather
         // than the registry itself. In effect this is intended to be a quite
         // cheap operation.
-        let summaries = match self.load_summaries(name, load)? {
-            Poll::Ready(summaries) => summaries,
-            Poll::Pending => return Poll::Pending,
-        };
+        let name = InternedString::new(name);
+        let summaries = ready!(self.load_summaries(name, load)?);
 
         // Iterate over our summaries, extract all relevant ones which match our
         // version requirement, and then parse all corresponding rows in the
@@ -367,68 +360,38 @@ impl<'cfg> RegistryIndex<'cfg> {
             .chars()
             .flat_map(|c| c.to_lowercase())
             .collect::<String>();
-        let raw_path = make_dep_path(&fs_name, false);
 
-        let mut any_pending = false;
-        // Attempt to handle misspellings by searching for a chain of related
-        // names to the original `raw_path` name. Only return summaries
-        // associated with the first hit, however. The resolver will later
-        // reject any candidates that have the wrong name, and with this it'll
-        // along the way produce helpful "did you mean?" suggestions.
-        for (i, path) in UncanonicalizedIter::new(&raw_path).take(1024).enumerate() {
-            let summaries = Summaries::parse(
-                root,
-                &cache_root,
-                path.as_ref(),
-                self.source_id,
-                load,
-                self.config,
-            )?;
-            if summaries.is_pending() {
-                if i == 0 {
-                    // If we have not herd back about the name as requested
-                    // then don't ask about other spellings yet.
-                    // This prevents us spamming all the variations in the
-                    // case where we have the correct spelling.
-                    return Poll::Pending;
-                }
-                any_pending = true;
-            }
-            if let Poll::Ready(Some(summaries)) = summaries {
-                self.summaries_cache.insert(name, summaries);
-                return Poll::Ready(Ok(self.summaries_cache.get_mut(&name).unwrap()));
-            }
-        }
-
-        if any_pending {
-            return Poll::Pending;
-        }
-
-        // If nothing was found then this crate doesn't exists, so just use an
-        // empty `Summaries` list.
-        self.summaries_cache.insert(name, Summaries::default());
+        let path = make_dep_path(&fs_name, false);
+        let summaries = ready!(Summaries::parse(
+            root,
+            &cache_root,
+            path.as_ref(),
+            self.source_id,
+            load,
+            self.config,
+        ))?
+        .unwrap_or_default();
+        self.summaries_cache.insert(name, summaries);
         Poll::Ready(Ok(self.summaries_cache.get_mut(&name).unwrap()))
     }
 
-    /// Clears the in-memory summmaries cache.
+    /// Clears the in-memory summaries cache.
     pub fn clear_summaries_cache(&mut self) {
         self.summaries_cache.clear();
     }
 
     pub fn query_inner(
         &mut self,
-        dep: &Dependency,
+        name: &str,
+        req: &OptVersionReq,
         load: &mut dyn RegistryData,
         yanked_whitelist: &HashSet<PackageId>,
         f: &mut dyn FnMut(Summary),
     ) -> Poll<CargoResult<()>> {
         if self.config.offline() {
-            match self.query_inner_with_online(dep, load, yanked_whitelist, f, false)? {
-                Poll::Ready(0) => {}
-                Poll::Ready(_) => return Poll::Ready(Ok(())),
-                Poll::Pending => return Poll::Pending,
-            }
-            // If offline, and there are no matches, try again with online.
+            // This should only return `Poll::Ready(Ok(()))` if there is at least 1 match.
+            //
+            // If there are 0 matches it should fall through and try again with online.
             // This is necessary for dependencies that are not used (such as
             // target-cfg or optional), but are not downloaded. Normally the
             // build should succeed if they are not downloaded and not used,
@@ -436,14 +399,20 @@ impl<'cfg> RegistryIndex<'cfg> {
             // then cargo will fail to download and an error message
             // indicating that the required dependency is unavailable while
             // offline will be displayed.
+            if ready!(self.query_inner_with_online(name, req, load, yanked_whitelist, f, false)?)
+                > 0
+            {
+                return Poll::Ready(Ok(()));
+            }
         }
-        self.query_inner_with_online(dep, load, yanked_whitelist, f, true)
+        self.query_inner_with_online(name, req, load, yanked_whitelist, f, true)
             .map_ok(|_| ())
     }
 
     fn query_inner_with_online(
         &mut self,
-        dep: &Dependency,
+        name: &str,
+        req: &OptVersionReq,
         load: &mut dyn RegistryData,
         yanked_whitelist: &HashSet<PackageId>,
         f: &mut dyn FnMut(Summary),
@@ -451,10 +420,7 @@ impl<'cfg> RegistryIndex<'cfg> {
     ) -> Poll<CargoResult<usize>> {
         let source_id = self.source_id;
 
-        let summaries = match self.summaries(dep.package_name(), dep.version_req(), load)? {
-            Poll::Ready(summaries) => summaries,
-            Poll::Pending => return Poll::Pending,
-        };
+        let summaries = ready!(self.summaries(name, req, load))?;
 
         let summaries = summaries
             // First filter summaries for `--offline`. If we're online then
@@ -469,7 +435,7 @@ impl<'cfg> RegistryIndex<'cfg> {
             // is not known here.
             .filter(|s| (online || load.is_crate_downloaded(s.summary.package_id())))
             // Next filter out all yanked packages. Some yanked packages may
-            // leak throguh if they're in a whitelist (aka if they were
+            // leak through if they're in a whitelist (aka if they were
             // previously in `Cargo.lock`
             .filter(|s| !s.yanked || yanked_whitelist.contains(&s.summary.package_id()))
             .map(|s| s.summary.clone());
@@ -479,7 +445,6 @@ impl<'cfg> RegistryIndex<'cfg> {
         // `<pkg>=<p_req>o-><f_req>` where `<pkg>` is the name of a crate on
         // this source, `<p_req>` is the version installed and `<f_req> is the
         // version requested (argument to `--precise`).
-        let name = dep.package_name().as_str();
         let precise = match source_id.precise() {
             Some(p) if p.starts_with(name) && p[name.len()..].starts_with('=') => {
                 let mut vers = p[name.len() + 1..].splitn(2, "->");
@@ -491,7 +456,7 @@ impl<'cfg> RegistryIndex<'cfg> {
         };
         let summaries = summaries.filter(|s| match &precise {
             Some((current, requested)) => {
-                if dep.version_req().matches(current) {
+                if req.matches(current) {
                     // Unfortunately crates.io allows versions to differ only
                     // by build metadata. This shouldn't be allowed, but since
                     // it is, this will honor it if requested. However, if not
@@ -531,7 +496,7 @@ impl<'cfg> RegistryIndex<'cfg> {
     ) -> Poll<CargoResult<bool>> {
         let req = OptVersionReq::exact(pkg.version());
         let found = self
-            .summaries(pkg.name(), &req, load)
+            .summaries(&pkg.name(), &req, load)
             .map_ok(|mut p| p.any(|summary| summary.yanked));
         found
     }
@@ -583,10 +548,7 @@ impl Summaries {
             Err(e) => log::debug!("cache missing for {:?} error: {}", relative, e),
         }
 
-        let response = match load.load(root, relative, index_version.as_deref())? {
-            Poll::Pending => return Poll::Pending,
-            Poll::Ready(response) => response,
-        };
+        let response = ready!(load.load(root, relative, index_version.as_deref())?);
 
         match response {
             LoadResponse::CacheValid => {
@@ -594,7 +556,6 @@ impl Summaries {
                 return Poll::Ready(Ok(cached_summaries));
             }
             LoadResponse::NotFound => {
-                debug_assert!(cached_summaries.is_none());
                 if let Err(e) = fs::remove_file(cache_path) {
                     if e.kind() != ErrorKind::NotFound {
                         log::debug!("failed to remove from cache: {}", e);
