@@ -133,6 +133,8 @@ pub fn registry_credential_config(
         secret_key_subject: Option<String>,
         #[serde(rename = "default")]
         _default: Option<String>,
+        #[serde(rename = "protocol")]
+        _protocol: Option<String>,
     }
 
     log::trace!("loading credential config for {}", sid);
@@ -182,7 +184,6 @@ pub fn registry_credential_config(
         let index = sid.canonical_url();
         let mut names: Vec<_> = config
             .env()
-            .iter()
             .filter_map(|(k, v)| {
                 Some((
                     k.strip_prefix("CARGO_REGISTRIES_")?
@@ -346,6 +347,8 @@ impl fmt::Display for AuthorizationErrorReason {
 pub struct AuthorizationError {
     /// Url that was attempted
     pub sid: SourceId,
+    /// The `registry.default` config value.
+    pub default_registry: Option<String>,
     /// Url where the user could log in.
     pub login_url: Option<Url>,
     /// Specific reason indicating what failed
@@ -355,9 +358,14 @@ impl Error for AuthorizationError {}
 impl fmt::Display for AuthorizationError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         if self.sid.is_crates_io() {
+            let args = if self.default_registry.is_some() {
+                " --registry crates-io"
+            } else {
+                ""
+            };
             write!(
                 f,
-                "{}, please run `cargo login`\nor use environment variable CARGO_REGISTRY_TOKEN",
+                "{}, please run `cargo login{args}`\nor use environment variable CARGO_REGISTRY_TOKEN",
                 self.reason
             )
         } else if let Some(name) = self.sid.alt_registry_key() {
@@ -420,6 +428,7 @@ pub fn auth_token(
         Some(token) => Ok(token.expose()),
         None => Err(AuthorizationError {
             sid: sid.clone(),
+            default_registry: config.default_registry()?,
             login_url: login_url.cloned(),
             reason: AuthorizationErrorReason::TokenMissing,
         }
@@ -579,7 +588,7 @@ pub enum Mutation<'a> {
         /// The version of the crate
         vers: &'a str,
     },
-    /// The user is attempting to unyank a crate.
+    /// The user is attempting to modify the owners of a crate.
     Owners {
         /// The name of the crate
         name: &'a str,
