@@ -1,31 +1,104 @@
 #![allow(unknown_lints)]
 
-use crate::core::{TargetKind, Workspace};
-use crate::ops::CompileOptions;
 use anyhow::Error;
-use cargo_util::ProcessError;
-use std::fmt;
+use curl::easy::Easy;
+use std::fmt::{self, Write};
 use std::path::PathBuf;
 
+use super::truncate_with_ellipsis;
+
 pub type CargoResult<T> = anyhow::Result<T>;
 
+/// These are headers that are included in error messages to help with
+/// diagnosing issues.
+pub const DEBUG_HEADERS: &[&str] = &[
+    // This is the unique ID that identifies the request in CloudFront which
+    // can be used for looking at the AWS logs.
+    "x-amz-cf-id",
+    // This is the CloudFront POP (Point of Presence) that identifies the
+    // region where the request was routed. This can help identify if an issue
+    // is region-specific.
+    "x-amz-cf-pop",
+    // The unique token used for troubleshooting S3 requests via AWS logs or support.
+    "x-amz-request-id",
+    // Another token used in conjunction with x-amz-request-id.
+    "x-amz-id-2",
+    // Whether or not there was a cache hit or miss (both CloudFront and Fastly).
+    "x-cache",
+    // The cache server that processed the request (Fastly).
+    "x-served-by",
+];
+
 #[derive(Debug)]
-pub struct HttpNot200 {
+pub struct HttpNotSuccessful {
     pub code: u32,
     pub url: String,
+    pub ip: Option<String>,
+    pub body: Vec<u8>,
+    pub headers: Vec<String>,
 }
 
-impl fmt::Display for HttpNot200 {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+impl HttpNotSuccessful {
+    pub fn new_from_handle(
+        handle: &mut Easy,
+        initial_url: &str,
+        body: Vec<u8>,
+        headers: Vec<String>,
+    ) -> HttpNotSuccessful {
+        let ip = handle.primary_ip().ok().flatten().map(|s| s.to_string());
+        let url = handle
+            .effective_url()
+            .ok()
+            .flatten()
+            .unwrap_or(initial_url)
+            .to_string();
+        HttpNotSuccessful {
+            code: handle.response_code().unwrap_or(0),
+            url,
+            ip,
+            body,
+            headers,
+        }
+    }
+
+    /// Renders the error in a compact form.
+    pub fn display_short(&self) -> String {
+        self.render(false)
+    }
+
+    fn render(&self, show_headers: bool) -> String {
+        let mut result = String::new();
+        let body = std::str::from_utf8(&self.body)
+            .map(|s| truncate_with_ellipsis(s, 512))
+            .unwrap_or_else(|_| format!("[{} non-utf8 bytes]", self.body.len()));
+
         write!(
-            f,
-            "failed to get 200 response from `{}`, got {}",
-            self.url, self.code
+            result,
+            "failed to get successful HTTP response from `{}`",
+            self.url
         )
+        .unwrap();
+        if let Some(ip) = &self.ip {
+            write!(result, " ({ip})").unwrap();
+        }
+        write!(result, ", got {}\n", self.code).unwrap();
+        if show_headers {
+            if !self.headers.is_empty() {
+                write!(result, "debug headers:\n{}\n", self.headers.join("\n")).unwrap();
+            }
+        }
+        write!(result, "body:\n{body}").unwrap();
+        result
     }
 }
 
-impl std::error::Error for HttpNot200 {}
+impl fmt::Display for HttpNotSuccessful {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str(&self.render(true))
+    }
+}
+
+impl std::error::Error for HttpNotSuccessful {}
 
 // =============================================================================
 // Verbose error
@@ -197,91 +270,6 @@ impl<'a> Iterator for ManifestCauses<'a> {
 
 impl<'a> ::std::iter::FusedIterator for ManifestCauses<'a> {}
 
-// =============================================================================
-// Cargo test errors.
-
-/// Error when testcases fail
-#[derive(Debug)]
-pub struct CargoTestError {
-    pub test: Test,
-    pub desc: String,
-    pub code: Option<i32>,
-    pub causes: Vec<ProcessError>,
-}
-
-impl fmt::Display for CargoTestError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        self.desc.fmt(f)
-    }
-}
-
-impl std::error::Error for CargoTestError {}
-
-#[derive(Debug)]
-pub enum Test {
-    Multiple,
-    Doc,
-    UnitTest {
-        kind: TargetKind,
-        name: String,
-        pkg_name: String,
-    },
-}
-
-impl CargoTestError {
-    pub fn new(test: Test, errors: Vec<ProcessError>) -> Self {
-        if errors.is_empty() {
-            panic!("Cannot create CargoTestError from empty Vec")
-        }
-        let desc = errors
-            .iter()
-            .map(|error| error.desc.clone())
-            .collect::<Vec<String>>()
-            .join("\n");
-        CargoTestError {
-            test,
-            desc,
-            code: errors[0].code,
-            causes: errors,
-        }
-    }
-
-    pub fn hint(&self, ws: &Workspace<'_>, opts: &CompileOptions) -> String {
-        match self.test {
-            Test::UnitTest {
-                ref kind,
-                ref name,
-                ref pkg_name,
-            } => {
-                let pkg_info = if opts.spec.needs_spec_flag(ws) {
-                    format!("-p {} ", pkg_name)
-                } else {
-                    String::new()
-                };
-
-                match *kind {
-                    TargetKind::Bench => {
-                        format!("test failed, to rerun pass '{}--bench {}'", pkg_info, name)
-                    }
-                    TargetKind::Bin => {
-                        format!("test failed, to rerun pass '{}--bin {}'", pkg_info, name)
-                    }
-                    TargetKind::Lib(_) => format!("test failed, to rerun pass '{}--lib'", pkg_info),
-                    TargetKind::Test => {
-                        format!("test failed, to rerun pass '{}--test {}'", pkg_info, name)
-                    }
-                    TargetKind::ExampleBin | TargetKind::ExampleLib(_) => {
-                        format!("test failed, to rerun pass '{}--example {}", pkg_info, name)
-                    }
-                    _ => "test failed.".into(),
-                }
-            }
-            Test::Doc => "test failed, to rerun pass '--doc'".into(),
-            _ => "test failed.".into(),
-        }
-    }
-}
-
 // =============================================================================
 // CLI errors
 
