@@ -1050,6 +1050,11 @@ fn decouple_proc_macro() {
 #[cargo_test]
 fn proc_macro_ws() {
     // Checks for bug with proc-macro in a workspace with dependency (shouldn't panic).
+    //
+    // Note, debuginfo is explicitly requested here to preserve the intent of this non-regression
+    // test: that will disable the debuginfo build dependencies optimization. Otherwise, it would
+    // initially trigger when the crates are built independently, but rebuild them with debuginfo
+    // when it sees the shared build/runtime dependency when checking the complete workspace.
     let p = project()
         .file(
             "Cargo.toml",
@@ -1057,6 +1062,9 @@ fn proc_macro_ws() {
             [workspace]
             members = ["foo", "pm"]
             resolver = "2"
+
+            [profile.dev.build-override]
+            debug = true
             "#,
         )
         .file(
@@ -1265,7 +1273,7 @@ fn resolver_bad_setting() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -1349,7 +1357,7 @@ fn resolver_not_both() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
