@@ -2,7 +2,7 @@
 
 use cargo::core::Edition;
 use cargo_test_support::compare::assert_match_exact;
-use cargo_test_support::git;
+use cargo_test_support::git::{self, init};
 use cargo_test_support::paths::{self, CargoPathExt};
 use cargo_test_support::registry::{Dependency, Package};
 use cargo_test_support::tools;
@@ -29,7 +29,7 @@ fn do_not_fix_broken_builds() {
     p.cargo("fix --allow-no-vcs")
         .env("__CARGO_FIX_YOLO", "1")
         .with_status(101)
-        .with_stderr_contains("[ERROR] could not compile `foo` due to previous error")
+        .with_stderr_contains("[ERROR] could not compile `foo` (lib) due to previous error")
         .run();
     assert!(p.read_file("src/lib.rs").contains("let mut x = 3;"));
 }
@@ -89,8 +89,14 @@ fn broken_fixes_backed_out() {
 
                 fn main() {
                     // Ignore calls to things like --print=file-names and compiling build.rs.
+                    // Also compatible for rustc invocations with `@path` argfile.
                     let is_lib_rs = env::args_os()
                         .map(PathBuf::from)
+                        .flat_map(|p| if let Some(p) = p.to_str().unwrap_or_default().strip_prefix("@") {
+                            fs::read_to_string(p).unwrap().lines().map(PathBuf::from).collect()
+                        } else {
+                            vec![p]
+                        })
                         .any(|l| l == Path::new("src/lib.rs"));
                     if is_lib_rs {
                         let path = PathBuf::from(env::var_os("OUT_DIR").unwrap());
@@ -652,7 +658,7 @@ fn fixes_missing_ampersand() {
         .with_stderr_contains("[FIXED] tests/a.rs (1 fix)")
         .with_stderr_contains("[FINISHED] [..]")
         .run();
-    p.cargo("build").run();
+    p.cargo("check").run();
     p.cargo("test").run();
 }
 
@@ -682,9 +688,9 @@ fn fix_features() {
         .build();
 
     p.cargo("fix --allow-no-vcs").run();
-    p.cargo("build").run();
+    p.cargo("check").run();
     p.cargo("fix --features bar --allow-no-vcs").run();
-    p.cargo("build --features bar").run();
+    p.cargo("check --features bar").run();
 }
 
 #[cargo_test]
@@ -765,6 +771,32 @@ commit the changes to these files:
     p.cargo("fix --allow-staged").run();
 }
 
+#[cargo_test]
+fn errors_about_untracked_files() {
+    let mut git_project = project().at("foo");
+    git_project = git_project.file("src/lib.rs", "pub fn foo() {}");
+    let p = git_project.build();
+    let _ = init(&p.root());
+
+    p.cargo("fix")
+        .with_status(101)
+        .with_stderr(
+            "\
+error: the working directory of this package has uncommitted changes, \
+and `cargo fix` can potentially perform destructive changes; if you'd \
+like to suppress this error pass `--allow-dirty`, `--allow-staged`, or \
+commit the changes to these files:
+
+  * Cargo.toml (dirty)
+  * src/ (dirty)
+
+
+",
+        )
+        .run();
+    p.cargo("fix --allow-dirty").run();
+}
+
 #[cargo_test]
 fn does_not_warn_about_clean_working_directory() {
     let p = git::new("foo", |p| p.file("src/lib.rs", "pub fn foo() {}"));
@@ -855,7 +887,7 @@ https://doc.rust-lang.org/edition-guide/editions/transitioning-an-existing-proje
     }
 
     p.cargo("fix --edition --allow-no-vcs")
-        .masquerade_as_nightly_cargo()
+        .masquerade_as_nightly_cargo(&["always_nightly"])
         .with_stderr(&format!(
             "\
 [CHECKING] foo [..]
@@ -901,15 +933,11 @@ fn prepare_for_latest_stable() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "fundamentally always nightly")]
 fn prepare_for_already_on_latest_unstable() {
     // During the period where a new edition is coming up, but not yet stable,
     // this test will check what happens if you are already on the latest. If
     // there is no next edition, it does nothing.
-    if !is_nightly() {
-        // This test is fundamentally always nightly.
-        return;
-    }
     let next_edition = match Edition::LATEST_UNSTABLE {
         Some(next) => next,
         None => {
@@ -936,7 +964,7 @@ fn prepare_for_already_on_latest_unstable() {
         .build();
 
     p.cargo("fix --edition --allow-no-vcs")
-        .masquerade_as_nightly_cargo()
+        .masquerade_as_nightly_cargo(&["always_nightly"])
         .with_stderr_contains("[CHECKING] foo [..]")
         .with_stderr_contains(&format!(
             "\
@@ -950,10 +978,6 @@ fn prepare_for_already_on_latest_unstable() {
 #[cargo_test]
 fn prepare_for_already_on_latest_stable() {
     // Stable counterpart of prepare_for_already_on_latest_unstable.
-    if !is_nightly() {
-        // Remove once 1.56 is stabilized.
-        return;
-    }
     if Edition::LATEST_UNSTABLE.is_some() {
         eprintln!("This test cannot run while the latest edition is unstable, skipping.");
         return;
@@ -1323,8 +1347,15 @@ fn fix_to_broken_code() {
                 use std::process::{self, Command};
 
                 fn main() {
+                    // Ignore calls to things like --print=file-names and compiling build.rs.
+                    // Also compatible for rustc invocations with `@path` argfile.
                     let is_lib_rs = env::args_os()
                         .map(PathBuf::from)
+                        .flat_map(|p| if let Some(p) = p.to_str().unwrap_or_default().strip_prefix("@") {
+                            fs::read_to_string(p).unwrap().lines().map(PathBuf::from).collect()
+                        } else {
+                            vec![p]
+                        })
                         .any(|l| l == Path::new("src/lib.rs"));
                     if is_lib_rs {
                         let path = PathBuf::from(env::var_os("OUT_DIR").unwrap());
@@ -1400,7 +1431,7 @@ fn fix_in_existing_repo_weird_ignore() {
     let p = git::new("foo", |project| {
         project
             .file("src/lib.rs", "")
-            .file(".gitignore", "foo\ninner\n")
+            .file(".gitignore", "foo\ninner\nCargo.lock\ntarget\n")
             .file("inner/file", "")
     });
 
@@ -1528,10 +1559,6 @@ fn rustfix_handles_multi_spans() {
 #[cargo_test]
 fn fix_edition_2021() {
     // Can migrate 2021, even when lints are allowed.
-    if !is_nightly() {
-        // Remove once 1.56 is stabilized.
-        return;
-    }
     let p = project()
         .file(
             "Cargo.toml",
@@ -1558,7 +1585,6 @@ fn fix_edition_2021() {
         )
         .build();
     p.cargo("fix --edition --allow-no-vcs")
-        .masquerade_as_nightly_cargo()
         .with_stderr(
             "\
 [CHECKING] foo v0.1.0 [..]
@@ -1715,7 +1741,7 @@ fn fix_with_run_cargo_in_proc_macros() {
             "src/lib.rs",
             r#"
                 use proc_macro::*;
-    
+
                 #[proc_macro]
                 pub fn foo(_input: TokenStream) -> TokenStream {
                     let output = std::process::Command::new(env!("CARGO"))
@@ -1725,7 +1751,7 @@ fn fix_with_run_cargo_in_proc_macros() {
                     eprintln!("{}", std::str::from_utf8(&output.stderr).unwrap());
                     println!("{}", std::str::from_utf8(&output.stdout).unwrap());
                     "".parse().unwrap()
-                }                    
+                }
             "#,
         )
         .file(
@@ -1740,7 +1766,6 @@ fn fix_with_run_cargo_in_proc_macros() {
         )
         .build();
     p.cargo("fix --allow-no-vcs")
-        .masquerade_as_nightly_cargo()
         .with_stderr_does_not_contain("error: could not find .rs file in rustc args")
         .run();
 }
@@ -1748,10 +1773,6 @@ fn fix_with_run_cargo_in_proc_macros() {
 #[cargo_test]
 fn non_edition_lint_migration() {
     // Migrating to a new edition where a non-edition lint causes problems.
-    if !is_nightly() {
-        // Remove once force-warn hits stable.
-        return;
-    }
     let p = project()
         .file("Cargo.toml", &basic_manifest("foo", "0.1.0"))
         .file(
@@ -1781,10 +1802,7 @@ fn non_edition_lint_migration() {
         .with_stderr_contains("[..]unused_imports[..]")
         .with_stderr_contains("[..]std::str::from_utf8[..]")
         .run();
-    p.cargo("fix --edition --allow-no-vcs")
-        // Remove once --force-warn is stabilized
-        .masquerade_as_nightly_cargo()
-        .run();
+    p.cargo("fix --edition --allow-no-vcs").run();
     let contents = p.read_file("src/lib.rs");
     // Check it does not remove the "unused" import.
     assert!(contents.contains("use std::str::from_utf8;"));
