@@ -5,11 +5,23 @@ use cargo_test_support::paths;
 use std::env;
 use std::fs::{self, File};
 
-fn create_empty_gitconfig() {
+fn create_default_gitconfig() {
     // This helps on Windows where libgit2 is very aggressive in attempting to
     // find a git config file.
     let gitconfig = paths::home().join(".gitconfig");
     File::create(gitconfig).unwrap();
+
+    // If we're running this under a user account that has a different default branch set up
+    // then tests that assume the default branch is master will fail. We set the default branch
+    // to master explicitly so that tests that rely on this behavior still pass.
+    fs::write(
+        paths::home().join(".gitconfig"),
+        r#"
+        [init]
+            defaultBranch = master
+        "#,
+    )
+    .unwrap();
 }
 
 #[cargo_test]
@@ -27,11 +39,17 @@ fn simple_lib() {
     let contents = fs::read_to_string(&lib).unwrap();
     assert_eq!(
         contents,
-        r#"#[cfg(test)]
+        r#"pub fn add(left: usize, right: usize) -> usize {
+    left + right
+}
+
+#[cfg(test)]
 mod tests {
+    use super::*;
+
     #[test]
     fn it_works() {
-        let result = 2 + 2;
+        let result = add(2, 2);
         assert_eq!(result, 4);
     }
 }
@@ -77,7 +95,24 @@ fn simple_git() {
 
     let fp = paths::root().join("foo/.gitignore");
     let contents = fs::read_to_string(&fp).unwrap();
-    assert_eq!(contents, "/target\nCargo.lock\n",);
+    assert_eq!(contents, "/target\n/Cargo.lock\n",);
+
+    cargo_process("build").cwd(&paths::root().join("foo")).run();
+}
+
+#[cargo_test(requires_hg)]
+fn simple_hg() {
+    cargo_process("new --lib foo --edition 2015 --vcs hg").run();
+
+    assert!(paths::root().is_dir());
+    assert!(paths::root().join("foo/Cargo.toml").is_file());
+    assert!(paths::root().join("foo/src/lib.rs").is_file());
+    assert!(paths::root().join("foo/.hg").is_dir());
+    assert!(paths::root().join("foo/.hgignore").is_file());
+
+    let fp = paths::root().join("foo/.hgignore");
+    let contents = fs::read_to_string(&fp).unwrap();
+    assert_eq!(contents, "^target$\n^Cargo.lock$\n",);
 
     cargo_process("build").cwd(&paths::root().join("foo")).run();
 }
@@ -88,8 +123,8 @@ fn no_argument() {
         .with_status(1)
         .with_stderr_contains(
             "\
-error: The following required arguments were not provided:
-    <path>
+error: the following required arguments were not provided:
+  <path>
 ",
         )
         .run();
@@ -303,9 +338,7 @@ fn subpackage_git_with_vcs_arg() {
 fn unknown_flags() {
     cargo_process("new foo --flag")
         .with_status(1)
-        .with_stderr_contains(
-            "error: Found argument '--flag' which wasn't expected, or isn't valid in this context",
-        )
+        .with_stderr_contains("error: unexpected argument '--flag' found")
         .run();
 }
 
@@ -362,7 +395,7 @@ fn new_default_edition() {
 #[cargo_test]
 fn new_with_bad_edition() {
     cargo_process("new --edition something_else foo")
-        .with_stderr_contains("error: 'something_else' isn't a valid value[..]")
+        .with_stderr_contains("error: invalid value 'something_else' for '--edition <YEAR>'")
         .with_status(1)
         .run();
 }
@@ -377,10 +410,6 @@ fn new_with_reference_link() {
 
 #[cargo_test]
 fn lockfile_constant_during_new() {
-    if !cargo_test_support::is_nightly() {
-        // Remove when 1.56 is stable (cargo new defaults to 2021).
-        return;
-    }
     cargo_process("new foo").run();
 
     cargo_process("build").cwd(&paths::root().join("foo")).run();
@@ -421,7 +450,7 @@ fn non_ascii_name() {
         .with_stderr(
             "\
 [WARNING] the name `Привет` contains non-ASCII characters
-Support for non-ASCII crate names is experimental and only valid on the nightly toolchain.
+Non-ASCII crate names are not supported by Rust.
 [CREATED] binary (application) `Привет` package
 ",
         )
@@ -475,7 +504,8 @@ or change the name in Cargo.toml with:
 #[cargo_test]
 fn git_default_branch() {
     // Check for init.defaultBranch support.
-    create_empty_gitconfig();
+    create_default_gitconfig();
+
     cargo_process("new foo").run();
     let repo = git2::Repository::open(paths::root().join("foo")).unwrap();
     let head = repo.find_reference("HEAD").unwrap();
@@ -494,3 +524,37 @@ fn git_default_branch() {
     let head = repo.find_reference("HEAD").unwrap();
     assert_eq!(head.symbolic_target().unwrap(), "refs/heads/hello");
 }
+
+#[cargo_test]
+fn non_utf8_str_in_ignore_file() {
+    let gitignore = paths::home().join(".gitignore");
+    File::create(gitignore).unwrap();
+
+    fs::write(paths::home().join(".gitignore"), &[0xFF, 0xFE]).unwrap();
+
+    cargo_process(&format!("init {} --vcs git", paths::home().display()))
+        .with_status(101)
+        .with_stderr(
+            "\
+error: Failed to create package `home` at `[..]`
+
+Caused by:
+  Character at line 0 is invalid. Cargo only supports UTF-8.
+",
+        )
+        .run();
+}
+
+#[cfg(unix)]
+#[cargo_test]
+fn path_with_invalid_character() {
+    cargo_process("new --name testing test:ing")
+        .with_stderr(
+            "\
+[WARNING] the path `[CWD]/test:ing` contains invalid PATH characters (usually `:`, `;`, or `\"`)
+It is recommended to use a different name to avoid problems.
+[CREATED] binary (application) `testing` package
+",
+        )
+        .run();
+}
